#!/usr/local/bin/ruby
# -*- coding: utf-8 -*-

if RUBY_VERSION.to_f <= 1.8
	$KCODE = 'u'
else
	Encoding.default_internal = 'utf-8'
	Encoding.default_external = 'utf-8'
end

ENV['GEM_HOME'] ||= '/home/utb/lib/ruby/gem'

require 'timeout'
require 'date'
require 'rubygems'
require 'htmlentities'
require 'cgi'

ROOT_URL = 'https://wikiwiki.jp/odohuran/'
CACHE_EXPIRES = 60
MAX_ANCHOR_LENGTH = 32
$page_name = '%E3%82%B4%E3%83%9F%E3%81%BF%E3%81%9F%E3%81%84%E3%81%AA%E5%A4%89%E6%8F%9B%E8%BE%9E%E6%9B%B8'
$cache_dir = File.dirname(__FILE__) + '/cache/'

def get_page_src()
	cache_file = $cache_dir + 'page_cache.html'
	if File.exist?(cache_file) && Time.now - File.stat(cache_file).mtime < CACHE_EXPIRES
		return File.read(cache_file)
	end
	data = `wget #{ROOT_URL}#{$page_name} -O -`
	raise "Page not found." if data.to_s.empty?
	Dir.mkdir($cache_dir) unless File.exist?($cache_dir)
	File.open(cache_file, mode = 'w') { |f|
		f.flock(File::LOCK_EX)
		f.write(data)
	}
	return data
end

def get_utf8text(anchor)
	raise "Anchor is too long." if MAX_ANCHOR_LENGTH < anchor.length
	cache = $cache_dir + anchor.gsub('[^0-9a-zA-Z_-]', '_')[0, MAX_ANCHOR_LENGTH] + '.dat'
	if File.exist?(cache) && Time.now - File.stat(cache).mtime < CACHE_EXPIRES
		return File.read(cache)
	end
	src = get_page_src()
	coder = HTMLEntities.new
	words = []
	phase = 0
	src.split(/(\n|\r|\r\n)/).each { |line|
		case phase
		when 0
			phase = 1 if line.include?("id=\"#{anchor}\"")
			next
		when 1
			next if ! line.include?('<pre>')
			line.gsub!('<pre>', '')
			phase = 2
		when 2
			break if line.include?('</pre>')
		end
		line.strip!
		next if line.empty?
		words << coder.decode(line)
	}
	raise "Anchor not found." if phase == 0
	raise 'PRE-tag not found.' if phase == 1
	words << ''
	result = words.join("\n")
	File.open(cache, mode = 'w') { |f|
		f.flock(File::LOCK_EX)
		f.write(result)
	}
	return result
end

################
# START HERE ! #
################
begin
	req = CGI.new
	if !(req.params['page'].empty?)
		$page_name = req.params['page'].to_s
		n = CGI.unescape($page_name)
		raise 'Bad page name.' if /[\?#:\.%]/i =~ n
		require 'digest'
		$cache_dir = File.dirname(__FILE__) + '/cache/' + Digest::MD5.hexdigest($page_name) + '/'
	end
	anchor = req['anchor'].empty? ? 'jisho' : req.params['anchor'].join(' ')
	Timeout.timeout(3) {
		result = ''
		anchor.split(/[\s\+]/).each { |a|
			result << get_utf8text(a)
		}
		print "Content-Type: application/octet-stream; charset=UTF-8\n\n"
		print result
	}
rescue => e
	print "Content-Type: text/plain; charset=UTF-8\n\n"
	print "ERROR SSU !\n"
	print e.message
	print "\n\n"
	print "Stack trace\n"
	print e.backtrace.join("\n")
end

